package form;

import form.io.*;
import form.data.*;
import form.comp.*;
import form.edit.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import java.applet.*;
import java.net.*;


public class Viewer {

	private static final String compPre = "comp";
	public boolean edit = false;
	private Vector panels = null;
	private Panel panel = null;
	private ScrollPane sPane = null;
	private FormPanel currFormPanel = null;
	private Data data = null;
	private Applet applet = null;
	private Panel[] pwpanel = null;

	private Frame frame = null;
	//a vector containing all components
	//this is only used for retrieving data
	//when for is submitted
	private Vector comps  = null;


	public void saveLocal(){
		//ask for location
		Save save = new Save(Save.LOCAL,frame);
		ToHtml toHtml = new ToHtml();
		toHtml(toHtml);
		String data = toHtml.getText();
		if (data== null){
			System.out.println("Data is null");
			return;
		}

		save.save(toHtml.getText());
	}

	public void saveFtp(){
		//ask for location
		Save save = new Save(Save.FTP,frame);
		ToHtml toHtml = new ToHtml();
		toHtml(toHtml);
		save.save(toHtml.getText());
	}

	private void toHtml(ToHtml toHtml){


	   toHtml.saveHeader(data.getPropertyData());

	   for (int i = 0; i < panels.size(); i++){
	      ((FormPanel)panels.elementAt(i)).save(toHtml,compPre+i);
	   }

	   data.save(toHtml);

	}

	public Viewer(boolean edi,Panel p,Open open, Frame frame,Applet applet){
	        edit = edi;
		panel = p;
		this.frame = frame;
		this.applet = applet;

		data = new Data(this,edit,frame);

		if (edit) {
			frame.setMenuBar(new FormMenu(this,data));
		}

		if (open != null){
		   open(open);
		}
		else {
		  newForm();
		}
	}

	public void newForm(){
	   //hide the current panel
	   if (currFormPanel!=null)
	     currFormPanel.setVisible(false);

	   panels = new Vector();
	   comps  = new Vector();

	   //add a blank one
	   panels.addElement(new FormPanel());
	   setViewable("root");

	   //see if this is needed
	   panel.validate();
	}
	   

	public void openLocal(){
	   Open open = new Open(Open.LOCAL,frame);
	   if (open.isOpen()){
	     open(open);
	   }
	   else {
	      System.out.println("Local open Failed");
	   }
	}

	public void openFtp(){
	   Open open = new Open(Open.FTP,frame);
	   if (open.isOpen()){
	     open(open);
	   }
	   else {
	      System.out.println("FTP open Failed");
	   }
	}

	public void open(Open open){
	   newForm();
	   data.open(open);
	   openComps(open);
	}

	public void openComps(Open open){

	   int count = 0;
	   int pCount = 0;
	   String pprefix = "";
           String        className = "";//the class to be added
	   Class	 newClass  = null;
	   FormInterface newObject = null;
	   FormActionListener listen = new FormActionListener(this);
	   panels = new Vector();

	   while ( (className = open.getParameter(compPre+pCount)) != null ){

	      try {

		FormPanel fp =  new FormPanel();
		pprefix = fp.open(open,compPre+(pCount));
		panels.addElement(fp);
		count = 0;
	       while ( (className = open.getParameter(pprefix+count+".")) != null ){

			//returns a Class for the named component
			newClass = Class.forName(className);
			//returns a new form object created with the empty constructor
			newObject = ((FormInterface)newClass.newInstance());


			if (!edit) newObject.addActionListener(listen);

			//sets the parameters for specific component
			newObject.open(open,edit,pprefix+count+".");
	
			((FormPanel)panels.lastElement()).add(newObject);
			comps.addElement(newObject);
			count++;
		}
	      } catch (ClassNotFoundException e){System.out.println("Can't find class : "+className);
	      } catch (Exception e) { e.printStackTrace();}
	      pCount ++;

	   }

	  if ( (!edit) && (!contains("Submit")) ) formFooter(listen);

          setViewable("root");
	}
	   
	private void formFooter(FormActionListener listen){

	   FormInterface newObject = ((FormInterface)comps.lastElement());

	   //the hardcoded footer for e-mail and submit
	   //the program won't wrok without them so here they are for ever

	   //passwords
	   
	   //set the last object to end a row
	   if (newObject != null)
	     newObject.getGridBagConstraints().gridwidth = 0;

	   //e-mail address and submit button
	   newObject = new FormLabel();
	   newObject.setText("E-mail");
	   ((FormPanel)panels.lastElement()).add(newObject);
	   newObject = new FormTextField();
	   newObject.setName("email");
	   newObject.setText("npib@vt.edu");
	   newObject.addActionListener(listen);
	   ((FormPanel)panels.lastElement()).add(newObject);

	   comps.addElement(newObject);

	   newObject = new FormButton("Submit",edit);
	   newObject.getGridBagConstraints().gridwidth = 0;
	   newObject.addActionListener(listen);
	   ((FormPanel)panels.lastElement()).add(newObject);
	}

	public void addPasswords(){
	   GridBagConstraints c = ((GridBagConstraints)new Constraints());
	   c.gridwidth = 0;
	   pwpanel = data.getPasswordPanels();
	   Panel pan = new Panel(new GridLayout(pwpanel.length,1));
	   for (int i =0; i < pwpanel.length;i++){
	      ((FormPanel)panels.lastElement()).add(pwpanel[i],c);
	   }
	}

	/**
	*@param String name Checks if a component of this name exists
	*/
	public FormInterface getComp(String name){

		for (int i =0; i < comps.size(); i++) 
			if (((FormInterface)comps.elementAt(i)).getName().equals(name) )
				return ((FormInterface)comps.elementAt(i));

		return null;
	}

	public String getComps(){

		String compList = "";
		for (int i =0; i < comps.size(); i++) {
		  compList +=((FormInterface)comps.elementAt(i)).getName()+" ";
		}

		return compList;
	}

	/**
	*@param String name Checks if a component of this name exists
	*/
	public boolean contains(String name){

		for (int i =0; i < comps.size(); i++) 
			if ( ((FormInterface)comps.elementAt(i)).getName().equals(name) )
				return true;

		return false;
	}

	/**
	*@param String name sets the panel with this name viewable
	* if name does not exist the default or root panel is displayed
	*/
	public FormPanel setViewable(String name){
		FormPanel curr = null;
        //should create a generic container so both pane and scroll panel are smoother

        if (edit && (sPane==null) ){
            sPane = new ScrollPane();
            sPane.setVisible(true);
            panel.add(sPane);
            panel.validate();
        }

		if (name.equals("root")){
			curr = ((FormPanel)panels.elementAt(0));
		}
		else if(name.equals("submit")){
		   currFormPanel.setVisible(false);
		    //panel.remove(currFormPanel);
		   panel.setSize(400,200);
		   panel.setLayout(new GridLayout(4,1));
		   panel.add(new Label("Submiting Job"));
		   //panel.add(new Label("This May take several minutes."));
		   //panel.add(new Label("Results will be displayed in a new window."));
		   //panel.add(new Label("They will also be mail to the provided address."));
		   panel.doLayout();
		   return null;
		}
		 	
		else {
			for (int i =0; i < panels.size(); i++)
			   if ( ((FormPanel)panels.elementAt(i)).getName().equals(name) )
			      curr = ((FormPanel)panels.elementAt(i));
		}

	 	if (currFormPanel != null){
		   currFormPanel.setVisible(false);

                  if (edit){
	             sPane.remove(currFormPanel);
	          }
		  else {
		    panel.remove(currFormPanel);
		  }
		}

		currFormPanel = curr;


                if (edit){
	           sPane.add(curr);
                }
                else {
		    panel.add(curr);
		    addPasswords();
                }
		currFormPanel.setVisible(true);

		curr.doLayout();
	        //curr.redo();
                return curr;
	}


	/**
	* @param FormInterface fi A new form interface object
	* this method is called from from the form menu.
	* It is used to insert new objects after the initial
	* creation
	*/
	public void add(FormInterface fi){

		currFormPanel.add(fi);
		comps.addElement(fi);
	}

	public void removeSelected(){
		currFormPanel.removeSelected();
	}

	public void submit(){
		Save save = new Save(Save.SUBMIT,frame);
		ToHtml toHtml = new ToHtml();

		toHtml(toHtml);
	        String email = getComp("email").getText();
		String name = data.getName();
		String destination = applet.getCodeBase().getProtocol()+"://"+applet.getCodeBase().getHost();
		setViewable("submit");
		save.submit(name,email,destination,data.getCgiLocation());
		String resultUrl = save.sendGet("html",toHtml.getSubmit(),applet.getAppletContext());
		save.sendGet("files",data.getFiles(),applet.getAppletContext());
		save.sendClose(getPasswords());
		//applet.setVisible(false);
//  		try {
//  		   applet.getAppletContext().showDocument(new URL(resultUrl+"/submitting.html"),"_blank");
//  		} catch (java.net.MalformedURLException e){System.err.println("can't open url");}
		applet.stop();
		applet.destroy();
	}

	private String getPasswords(){
		String pass= "";

		int pnum = pwpanel.length;
		PasswordPanel[] p = (PasswordPanel[])pwpanel;
		for (int i = 0; i < pnum; i++){
			pass+="&id="+p[i].getId();
			pass+="&user="+p[i].getUser();
			pass+="&pass="+p[i].getPass();
			pass+="&host="+p[i].getHost();
		}
		return pass;
	}

	public void refresh(){
	   currFormPanel.redo();
	}
}
